/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'ovl_create_or_link' function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When process calls 'ovl_create_or_link', we need to correctly handle
 *    this situation and adjust 'off' flag
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 28.III.2019
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../../p_lkrg_main.h"

char p_ovl_create_or_link_kretprobe_state = 0x0;

static struct kretprobe p_ovl_create_or_link_kretprobe = {
    .kp.symbol_name = "ovl_create_or_link",
    .handler = p_ovl_create_or_link_ret,
    .entry_handler = p_ovl_create_or_link_entry,
    .data_size = sizeof(struct p_ovl_create_or_link_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_ovl_create_or_link_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_ovl_create_or_link_entry>\n");

   p_debug_kprobe_log(
          "Leaving function <p_ovl_create_or_link_entry>\n");

   return 0;
}


int p_ovl_create_or_link_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_ovl_create_or_link_ret>\n");

   if (p_is_ed_task(current)) {
      // Update process
      spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
      if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
         if (p_verify_ovl_create_or_link(p_tmp))
            p_set_ed_process_override_on(p_tmp);
      }
      p_validate_task_f(current);
      if (p_ed_enforce_pcfi(current, p_tmp, p_regs)) {
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
                current->comm,task_pid_nr(current));
         p_ed_kill_task_by_task(current);
      }
      spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);
   }

   p_debug_kprobe_log(
          "Leaving function <p_ovl_create_or_link_ret>\n");

   return 0x0;
}


int p_install_ovl_create_or_link_hook(void) {

   int p_ret;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_ovl_create_or_link_hook>\n");

   if ( (p_ret = register_kretprobe(&p_ovl_create_or_link_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_ovl_create_or_link_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_uninstall_ovl_create_or_link_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_ovl_create_or_link_kretprobe.kp.symbol_name,
               p_ovl_create_or_link_kretprobe.kp.addr);
   p_ovl_create_or_link_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_uninstall_ovl_create_or_link_hook_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_ovl_create_or_link_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_ovl_create_or_link_hook(void) {

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_ovl_create_or_link_hook>\n");

   if (!p_ovl_create_or_link_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_ovl_create_or_link_kretprobe.kp.symbol_name,
                  p_ovl_create_or_link_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_ovl_create_or_link_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_ovl_create_or_link_kretprobe.kp.symbol_name,
                  p_ovl_create_or_link_kretprobe.kp.addr,
                  p_ovl_create_or_link_kretprobe.nmissed);
      p_ovl_create_or_link_kretprobe_state = 0x0;
   }

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_ovl_create_or_link_hook>\n");
}
