/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'lookup_fast' function
 *
 * Notes:
 *  - Enforce Exploit Detection CFI
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 20.XI.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../p_lkrg_main.h"


char p_pcfi_lookup_fast_kretprobe_state = 0x0;

static struct kretprobe p_pcfi_lookup_fast_kretprobe = {
    .kp.symbol_name = "lookup_fast",
    .handler = p_pcfi_lookup_fast_ret,
    .entry_handler = p_pcfi_lookup_fast_entry_extra,
    .data_size = sizeof(struct p_pcfi_lookup_fast_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_pcfi_lookup_fast_entry_noextra(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_lookup_fast_entry>\n");

//   p_ed_enforce_validation();

   if (p_is_ed_task(current)) {
      spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
      p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current));
      if (p_ed_pcfi_validate_sp(current, p_tmp, p_regs_get_sp(p_regs))) {
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
                current->comm,task_pid_nr(current));
         // kill this process!
         p_ed_kill_task_by_task(current);
      }
      spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);
   }

   p_ed_pcfi_validate_cpu(1);

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_lookup_fast_entry>\n");

   return 0x0;
}
int p_pcfi_lookup_fast_entry_extra(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_lookup_fast_entry>\n");

//   p_ed_enforce_validation();

   if (p_is_ed_task(current)) {
      spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
      p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current));
      if (p_ed_pcfi_validate_sp(current, p_tmp, p_regs_get_sp(p_regs))) {
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
                current->comm,task_pid_nr(current));
         // kill this process!
         p_ed_kill_task_by_task(current);
      }
      spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);
   }

   p_ed_pcfi_validate_cpu_extra(1);

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_lookup_fast_entry>\n");

   return 0x0;
}


int p_pcfi_lookup_fast_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_lookup_fast_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_lookup_fast_ret>\n");

   return 0x0;
}


int p_install_pcfi_lookup_fast_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_pcfi_lookup_fast_hook>\n");

   if (unlikely(!p_pcfi_CPU_flags)) {
      p_pcfi_lookup_fast_kretprobe.entry_handler = p_pcfi_lookup_fast_entry_noextra;
   }

   if ( (p_ret = register_kretprobe(&p_pcfi_lookup_fast_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_pcfi_lookup_fast_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_pcfi_lookup_fast_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_pcfi_lookup_fast_kretprobe.kp.symbol_name,
               p_pcfi_lookup_fast_kretprobe.kp.addr);
   p_pcfi_lookup_fast_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_pcfi_lookup_fast_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_pcfi_lookup_fast_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_pcfi_lookup_fast_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_pcfi_lookup_fast_hook>\n");

   if (!p_pcfi_lookup_fast_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_pcfi_lookup_fast_kretprobe.kp.symbol_name,
                  p_pcfi_lookup_fast_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_pcfi_lookup_fast_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_pcfi_lookup_fast_kretprobe.kp.symbol_name,
                  p_pcfi_lookup_fast_kretprobe.kp.addr,
                  p_pcfi_lookup_fast_kretprobe.nmissed);
      p_pcfi_lookup_fast_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_pcfi_lookup_fast_hook>\n");
}
