/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept *call_usermodehelper* function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When kernel calls user-mode helper, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.II.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_call_usermodehelper_kretprobe_state = 0x0;

static struct kretprobe p_call_usermodehelper_kretprobe = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,3,0)
    .kp.symbol_name = "____call_usermodehelper",
#else
    .kp.symbol_name = "call_usermodehelper_exec_async",
#endif
    .handler = p_call_usermodehelper_ret,
    .entry_handler = p_call_usermodehelper_entry,
    .data_size = sizeof(struct p_call_usermodehelper_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

int p_call_usermodehelper_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   struct subprocess_info *p_subproc = (struct subprocess_info *)p_regs_get_arg1(p_regs);
   unsigned int p_cnt, p_umh_cnt;
   unsigned char p_umh_allowed = 0x0;
   unsigned long p_flags;
   struct p_umh_whitelist p_umh[] = {

      { "/etc/acpi/events/RadioPower.sh", 0x0 },
      { "/etc/acpi/wireless-rtl-ac-dc-power.sh", 0x0 },
      { "/sbin/pnpbios", 0x0 },
      { "/usr/sbin/eppfpga", 0x0 },
      { "/sbin/critical_overtemp", 0x0 },
      { "/sbin/bridge-stp", 0x0 },
      { "/sbin/nfsd-recall-failed", 0x0 },
      { "/sbin/poweroff", 0x0 },
      { "/sbin/modprobe", 0x0 },
      { "/sbin/ocfs2_hb_ctl", 0x0 },
      { "/sbin/nfsdcltrack", 0x0 },
      { "/sbin/nfs_cache_getent", 0x0 },
      { "/sbin/v86d", 0x0 },
      { "/sbin/drbdadm", 0x0 },
      { "/sbin/hotplug", 0x0 },
      { "/sbin/tomoyo-init", 0x0 },
   #if LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)
    #ifdef ANDROID_BUILD
      { "/system/bin/unififw", 0x0 },
    #else
      { "/usr/sbin/unififw", 0x0 },
    #endif /* ANDROID_BUILD */
   #elif LINUX_VERSION_CODE < KERNEL_VERSION(4,18,0)
      { "/usr/lib/lustre/lnet_upcall", 0x0 },
      { "/usr/lib/lustre/lnet_debug_log_upcall", 0x0 },
   #endif
   #if LINUX_VERSION_CODE < KERNEL_VERSION(4,12,0)
      { "/sbin/osd_login", 0x0 },
   #endif
      { "/system/bin/start", 0x0 },
      { "/lib/systemd/systemd-cgroups-agent", 0x0 },
      { "/usr/lib/systemd/systemd-cgroups-agent", 0x0 },
      { "/usr/share/apport/apport", 0x0 }

   };

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_entry>\n");

   p_ed_enforce_validation();

   if (p_lkrg_global_ctrl.p_umh_lock)
      goto p_call_usermodehelper_entry_not_allowed;

   spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);

   p_umh_cnt = sizeof(p_umh) / sizeof(p_umh[0]);
   for (p_cnt = 0x0; p_cnt < p_umh_cnt; p_cnt++) {
      if (p_umh_global[p_cnt].p_enabled) {
         if (!strcmp(p_umh[p_cnt].p_path,p_subproc->path)) {
            p_umh_allowed = 0x1;
            break;
         }
      }
   }

#if defined(CONFIG_STATIC_USERMODEHELPER)
   if (!strcmp(CONFIG_STATIC_USERMODEHELPER_PATH,p_subproc->path)) {
      p_umh_allowed = 0x1;
   }
#endif

   if (!p_umh_allowed) {
p_call_usermodehelper_entry_not_allowed:
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,18,0)
      if (!strcmp("none",p_subproc->path) && p_subproc->file) {
         p_print_log(P_LKRG_ERR,
                "<Exploit Detection> UMH is executing file from memory...\n");
      } else {
#endif
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> !!! BLOCKING UMH !!!\n");
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> Someone is trying to execute file: [%s]\n",
                p_subproc->path);
         p_print_log(P_LKRG_CRIT,
                "<Exploit Detection> --- . ---\n");
         memset((char *)p_subproc->path, '/', strlen(p_subproc->path));
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,18,0)
      }
#endif
   }

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_entry>\n");

   return 0;
}


int p_call_usermodehelper_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_ret>\n");

   // Update process
   spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
      p_update_ed_process(p_tmp, current);
      p_set_ed_process_on(p_tmp);
   }
   spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_ret>\n");

   return 0x0;
}


int p_install_call_usermodehelper_hook(void) {

   int p_ret;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_call_usermodehelper_hook>\n");

   if ( (p_ret = register_kretprobe(&p_call_usermodehelper_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_uninstall_call_usermodehelper_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_call_usermodehelper_kretprobe.kp.symbol_name,
               p_call_usermodehelper_kretprobe.kp.addr);
   p_call_usermodehelper_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_uninstall_call_usermodehelper_hook_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_call_usermodehelper_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_call_usermodehelper_hook(void) {

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_call_usermodehelper_hook>\n");

   if (!p_call_usermodehelper_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_call_usermodehelper_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_call_usermodehelper_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_call_usermodehelper_kretprobe.kp.addr,
                  p_call_usermodehelper_kretprobe.nmissed);
      p_call_usermodehelper_kretprobe_state = 0x0;
   }

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_call_usermodehelper_hook>\n");
}
