/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'wake_up_new_task' function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When process forks, child must be tracked as well! We need to update RB tree.
 *
 * Caveats:
 *  - Previous version was hooking 'do_fork' function.
 *
 * Timeline:
 *  - Created: 18.IX.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_wake_up_new_task_kretprobe_state = 0x0;

static struct kretprobe p_wake_up_new_task_kretprobe = {
    .kp.symbol_name = "wake_up_new_task",
    .handler = p_wake_up_new_task_ret,
    .entry_handler = p_wake_up_new_task_entry,
    .data_size = sizeof(struct p_wake_up_new_task_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_wake_up_new_task_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct task_struct *p_task = (struct task_struct *)p_regs_get_arg1(p_regs);
   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   if (p_task) {
      spin_lock_irqsave(&p_rb_ed_pids_lock, p_flags);
//      if (!(p_task->flags & PF_KTHREAD || is_global_init(p_task))) {
      if (p_is_ed_task(p_task)) {
         int p_ret;

         if ( (p_ret = p_dump_task_f(p_task)) != 0) {
            if (p_ret == 1) {
               // Update process
               if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(p_task))) != NULL) {
                  // This process is on the ED list - update information!
                  p_print_log(P_LKRG_INFO, "<wake_up_new_task> Updating ED pid[%d]\n",task_pid_nr(p_task));
                  p_update_ed_process(p_tmp, p_task);
                  p_reset_ed_flags(p_tmp);
               }
            } else {
               p_print_log(P_LKRG_WARN,
                      "<Exploit Detection> Error[%d] when trying to add process[%d |%s] for tracking!\n",
                                                                p_ret, task_pid_nr(p_task), p_task->comm);
            }
         }
      }
      spin_unlock_irqrestore(&p_rb_ed_pids_lock, p_flags);
   }

   p_ed_enforce_validation();

   return 0x0;
}


int p_wake_up_new_task_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_wake_up_new_task_ret>\n");

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_wake_up_new_task_ret>\n");

   return 0x0;
}


int p_install_wake_up_new_task_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_wake_up_new_task_hook>\n");

   if ( (p_ret = register_kretprobe(&p_wake_up_new_task_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_wake_up_new_task_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_wake_up_new_task_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_wake_up_new_task_kretprobe.kp.symbol_name,
               p_wake_up_new_task_kretprobe.kp.addr);
   p_wake_up_new_task_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_wake_up_new_task_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_wake_up_new_task_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_wake_up_new_task_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_wake_up_new_task_hook>\n");

   if (!p_wake_up_new_task_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_wake_up_new_task_kretprobe.kp.symbol_name,
                  p_wake_up_new_task_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_wake_up_new_task_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_wake_up_new_task_kretprobe.kp.symbol_name,
                  p_wake_up_new_task_kretprobe.kp.addr,
                  p_wake_up_new_task_kretprobe.nmissed);
      p_wake_up_new_task_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_wake_up_new_task_hook>\n");
}
